local ADDON_NAME, LWGD = ...
local MEDIA_FOLDER = LWGD.MEDIA_FOLDER
local VERSION = GetAddOnMetadata(ADDON_NAME,"Version")
local ObserversList = {}
local group = {}
local newgroup = {}

local IsMaster = true
local Master = ""


--#region helper functions --
function send_addon_check(target)
    if target == nil or not UnitIsConnected(target) then
        return 
    end 
    name = UnitName("player")
    C_ChatInfo.SendAddonMessage("LWGD", "CHECK_" .. name .. "_" .. VERSION, "WHISPER", target)
end

function broadcast_addon_check(channel)
    if channel == nil or channel == "solo" then
        return 
    end 
    local name = UnitName("player")
    C_ChatInfo.SendAddonMessage("LWGD", "CHECK_" .. name .. "_" .. VERSION, channel)
end

function send_play(sound)
    if sound == nil or not IsMaster then
        return 
    end

    local channel = (LWGD.Tools.GetGroupType())

    if channel ~= "solo" then
        C_ChatInfo.SendAddonMessage("LWGD", "PLAY_" .. sound , channel)
    else
        PlaySoundFile(MEDIA_FOLDER .. sound, "Master")
    end
end

function set_new_master()
    local tbl = LWGD.Tools.SortTable(ObserversList)
    local new_tbl = {}

    -- Make sure every observer is still connected
    for i,k in pairs(tbl) do
        local t_name = ObserversList[k].Name
        local t_version = ObserversList[k].Version
        if UnitIsConnected(t_name) then
            tinsert(new_tbl, { Name = t_name, Version = t_version})
        end
    end
    ObserversList = new_tbl

    -- If not in a solo group/not unsupported
    if (LWGD.Tools.GetGroupType()) ~= "solo" then
        local new_master = nil
        
        -- Try get raidleader as master
        for i,k in pairs(ObserversList) do
            local name = ObserversList[i].Name

            raidIndex = UnitInRaid(name)
            if raidIndex ~= nil then
                _, rank, subgroup = GetRaidRosterInfo(raidIndex);
               
                if rank == 2 and subgroup <= 4 then
                    new_master = name
                    break
                end
            end
        end

        -- Try get a player from Group 1-4 as master
        if new_master == nil then
            for i,k in pairs(ObserversList) do
                local name = ObserversList[i].Name
                raidIndex = UnitInRaid(name)
                if raidIndex ~= nil then
                    _, rank, subgroup = GetRaidRosterInfo(raidIndex);
                   
                    if subgroup <= 4 then
                        new_master = name
                        break
                    end
                else
                    -- If unit is not in a raid, but the Group leader
                    if UnitIsGroupLeader(name,"LE_PARTY_CATEGORY_HOME") then
                        new_master = name
                        break
                    end
                end
            end
        end

        if new_master ~= nil then
            Master = new_master
        else
            Master = UnitName("player")
        end
    else
        Master = UnitName("player")
    end

    if Master == UnitName("player") then
        IsMaster = true
    else
        IsMaster = false
    end
end

local function check_player(self, event)
    local timestamp, subevent, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = CombatLogGetCurrentEventInfo()

    if subevent == "UNIT_DIED" and IsMaster then
        destName = LWGD.Tools.GetNameWithoutRealm(destName)
        if destName == nil then return end
        if not UnitIsPlayer(destName) then return end

        local isMatch, name = LWGD.CheckPlayer(destName)
        local sound = LWGD.GetSound(name)
        send_play(sound)
    end
end

--#endregion helper functions --

LWGD.eventListener = CreateFrame("Frame", "LWGDEvents")
LWGD.eventListener:RegisterEvent("ADDON_LOADED")
LWGD.eventListener:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
LWGD.eventListener:RegisterEvent("PLAYER_LOGOUT")
LWGD.eventListener:RegisterEvent("PLAYER_LOGIN")
LWGD.eventListener:RegisterEvent("GROUP_ROSTER_UPDATE")
LWGD.eventListener:RegisterEvent("CHAT_MSG_ADDON")

LWGD.PlaySound = send_play

local DB = {
    UsedNames = UsedNamesList
}

local function SlashCommands(msg, editbox)
    if msg == 'list' then
        print("Total players with LWGD: " .. table.getn(ObserversList))

        for i, v in pairs(ObserversList) do
            local name = ObserversList[i].Name
            if name == Master then
                print("|cff22ff22Master|r", ObserversList[i].Name, (ObserversList[i].Version ~= nil and ObserversList[i].Version ~= "-") and "[" .. ObserversList[i].Version.. "]" or "[unknown version]")
            else
                print("|cffff2222Observer|r", ObserversList[i].Name, (ObserversList[i].Version ~= nil and ObserversList[i].Version ~= "-") and "[" .. ObserversList[i].Version .. "]" or "[unknown version]")
            end
        end
    elseif msg == 'names' then
        print("Showing all supported names and their soundfiles")
        for i, v in pairs(LWGD.Players) do
            print("|cff22ff22" .. i .. "|r [".. table.getn(LWGD.Players[i].Sounds) .. " sound file(s)]")
        end
    end
  end
  
  SLASH_LWGD1 = '/lwgd'
  
SlashCmdList["LWGD"] = SlashCommands

LWGD.eventListener:SetScript("OnEvent",
function(self, event, ...)
    if event == "ADDON_LOADED" then
        local name = ...
        if name == "LudwigGoesDown" then
            C_ChatInfo.RegisterAddonMessagePrefix("LWGD");
            LWGD.CreateOptions()
        end
    elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
        check_player(event)
    elseif event == "PLAYER_LOGOUT" then
    elseif event == "PLAYER_LOGIN" then
        local grp = (LWGD.Tools.GetGroupType())

        if ObserversList ~= nil then
            wipe(ObserversList)
        end

        broadcast_addon_check(grp)

    elseif event == "GROUP_ROSTER_UPDATE" then
        local grp, count = LWGD.Tools.GetGroupType()
        if grp ~= "solo" then

            -- Check if Player (self) is missing from the list
            if grp == "party" then
                local found = LWGD.Tools.ListHasValue(ObserversList, (UnitName("player")))
                -- Add Player if missing
                if not found then
                    tinsert(ObserversList, {Name = (UnitName("player")), Version = VERSION})
                    print("added self")
                end
            end
            
            -- Check for changes to the group
            group, newgroup = newgroup, group
            wipe(newgroup)
            for i = 1, count do
                local name = GetUnitName(grp..i, true)
                if name and name ~= UNKNOWN then
                    newgroup[name] = i
                end
            end

            -- Check for players who left the group
            for name, index in pairs(group) do
                if not newgroup[name] then                
                    for i, v in pairs(ObserversList) do
                        if v.Name == name then
                            tremove(ObserversList, i)
                            break
                        end
                    end
                end
            end

            for i, v in pairs(ObserversList) do
                if v.Name == UnitName("player") then
                    v.Version = VERSION
                    break
                end
            end

            broadcast_addon_check(grp)
        else    
            if ObserversList ~= nil then
                wipe(ObserversList)
            end

            IsMaster = true
        end

    elseif event == "CHAT_MSG_ADDON" then
        local prefix, text, _, message_sender = ...
        if prefix == "LWGD" then
            if LWGD.Tools.StringStartsWith(text, "CHECK_") then
                local myName = UnitName("player")
                local sender, version = strsplit("_", strsub(text, 7))
                local name_without_realm = LWGD.Tools.GetNameWithoutRealm(sender)

                if name_without_realm ~= nil then
                    if not LWGD.Tools.ListHasValue(ObserversList, name_without_realm) then
                        tinsert(ObserversList, { Name = name_without_realm, Version = version })
                        set_new_master()
                    end
                end
                C_ChatInfo.SendAddonMessage("LWGD", "ACK_" .. myName .. "_" .. VERSION, "WHISPER", message_sender)

            elseif LWGD.Tools.StringStartsWith(text, "ACK_") then
                local sender, version = strsplit("_", strsub(text, 5))
                sender = LWGD.Tools.GetNameWithoutRealm(sender)
                local found = LWGD.Tools.ListHasValue(ObserversList, sender)

                if not found then
                    tinsert(ObserversList, { Name = sender, Version = version })
                else
                    for i, v in pairs(ObserversList) do
                        if v.Name == sender then
                            v.Version = version
                            break
                        end
                    end
                end

                set_new_master()

            elseif LWGD.Tools.StringStartsWith(text, "PLAY_") then
                local sound = strsub(text, 6)
                local name_without_realm = LWGD.Tools.GetNameWithoutRealm(message_sender)

                if name_without_realm ~= nil then
                    if name_without_realm == Master then
                        PlaySoundFile(MEDIA_FOLDER .. sound, "Master")
                    end
                end
            end    
        end
    end
end)
