local ADDON_NAME, LWGD = ...

-- String: the original string
-- Start: the string to look for
local function startsWith(string, start)
    if start == nil then
        return false
    end

    return string.sub(string, 1, string.len(start)):lower() == start:lower()
end

local function has_value(list, value)
    for _, v in pairs(list) do
        if v.Name == value then
            return true
        end
    end
    return false
end
local function has_key(list, key)
    for i, _ in pairs(list) do
        if i == key then
            return true
        end
    end
    return false
end

local function get_group_type()
    if not IsInGroup("LE_PARTY_CATEGORY_HOME") then
        return "solo"
    end

    local n, unit = GetNumGroupMembers(), "raid"
    if not IsInRaid() then
        n, unit = n - 1, "party"
    end

    return unit, n
end

local sortFunction = function(a, b) return a < b end
function sort_table_by_value(tbl)

    local keys = {}
    for key in pairs(tbl) do
      table.insert(keys, key)
    end
  
    table.sort(keys, function(a, b)
      return sortFunction(tbl[a].Name, tbl[b].Name)
    end)
  
    return keys
end

local function get_name_without_realm(name)
    if name == nil then return "" end
    
    local dash = (string.find(name, '-', 1, true))
    if dash ~= nil then
        return strsub(name, 1, dash - 1)
    else
        return name
    end
end

LWGD.Tools = {}
LWGD.Tools.StringStartsWith = startsWith
LWGD.Tools.ListHasValue = has_value
LWGD.Tools.ListHasKey = has_key
LWGD.Tools.SortTable = sort_table_by_value
LWGD.Tools.GetGroupType = get_group_type
LWGD.Tools.GetNameWithoutRealm = get_name_without_realm